%
%
%  Smooth time Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="SmoothTime" icon="Matlab" tooltip="Smooth the intensity of each voxel along the time dimension using a gaussian filter.">
%          <Command>MatlabXT::XTSmoothTime(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%   
%   Smooth the intensity of each voxel along the time dimension using a 
%       gaussian filter. 
%
%

function XTSmoothTime(aImarisApplicationID, aSigma, aChannelIndex)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end
vImarisApplication.DataSetPushUndo('Smooth Time');

% get the data set 
vImarisDataSet = vImarisApplication.GetDataSet.Clone;

vChannelIndex = 1;
for vChannelId = 1:vImarisDataSet.GetSizeC
  if vImarisApplication.GetChannelVisibility(vChannelId-1) == true
    vChannelIndex = vChannelId;
    break;
  end
end

if nargin<3
    vSigma = 1;
    vAnswer = inputdlg({'Channel','Filter Width'}, 'Smooth Time', ...
        1, {num2str(vChannelIndex),num2str(vSigma)});
    if isempty(vAnswer)
        return;
    else
        vChannelIndex = str2double(char(vAnswer(1)));
        vSigma = str2double(char(vAnswer(2)));
    end
else
    vChannelIndex = aChannelIndex;
    vSigma = aSigma;
end

vImageProcessing = vImarisApplication.GetImageProcessing;

vNumberOfTimePoints = vImarisDataSet.GetSizeT;
vNumberOfSlices = vImarisDataSet.GetSizeZ;

vProgressDisplay = waitbar(0,'Smoothing Time');

% prepare the dataset 
vDataSet = vImarisApplication.GetFactory.CreateDataSet;
vDataSet.Create(vImarisDataSet.GetType, vImarisDataSet.GetSizeX, vImarisDataSet.GetSizeY, ...
    vNumberOfTimePoints, 1, 1);

vDataSet.SetExtendMinX(0);
vDataSet.SetExtendMinY(0);
vDataSet.SetExtendMinZ(0);
% gaussian filter does not operate along X and Y axis
vDataSet.SetExtendMaxX(5000*vImarisDataSet.GetSizeX);
vDataSet.SetExtendMaxY(5000*vImarisDataSet.GetSizeY);
vDataSet.SetExtendMaxZ(vNumberOfTimePoints);

for vSlice = 1:vNumberOfSlices
    % sets the time slices in the Z dimension
    for vTime = 1:vNumberOfTimePoints
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vData = vImarisDataSet.GetDataSliceBytes(vSlice-1, vChannelIndex-1, vTime-1);
            vDataSet.SetDataSliceBytes(vData, vTime-1, 0, 0);
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vData = vImarisDataSet.GetDataSliceShorts(vSlice-1, vChannelIndex-1, vTime-1);
            vDataSet.SetDataSliceShorts(vData, vTime-1, 0, 0);
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vData = vImarisDataSet.GetDataSliceFloats(vSlice-1, vChannelIndex-1, vTime-1);
            vDataSet.SetDataSliceFloats(vData, vTime-1, 0, 0);
        end
    end

    vImageProcessing.GaussFilterChannel(vDataSet, 0, vSigma);
    
    for vTime = 1:vNumberOfTimePoints
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vData = vDataSet.GetDataSliceBytes(vTime-1, 0, 0);
            vImarisDataSet.SetDataSliceBytes(vData, vSlice-1, vChannelIndex-1, vTime-1);
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vData = vDataSet.GetDataSliceShorts(vTime-1, 0, 0);
            vImarisDataSet.SetDataSliceShorts(vData, vSlice-1, vChannelIndex-1, vTime-1);
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vData = vDataSet.GetDataSliceFloats(vTime-1, 0, 0);
            vImarisDataSet.SetDataSliceFloats(vData, vSlice-1, vChannelIndex-1, vTime-1);
        end
    end

    waitbar(vSlice/vNumberOfSlices)
end
close(vProgressDisplay);

vImarisApplication.SetDataSet(vImarisDataSet);
